@@ -177,13 +177,8 @@ unsigned char *line;
 unsigned char *tmpline;
 unsigned char token[64];
 unsigned char notdef[]="/.notdef";
-#ifdef Omega
-unsigned char grid[65536];
-unsigned char tmpgrid[65536];
-#else
 unsigned char grid[256];
 unsigned char tmpgrid[256];
-#endif
 unsigned char psfontfile[500]; /* really should dynamically allocate */
 unsigned char psvectfile[500];
 unsigned char basevect[]="ad.enc";
